@@ -233,13 +233,19 @@ cb_ui_theme_selection_changed (GtkTreeSelection *selection)
 }
 
 static void
-cb_xfwm4_sync_switch_toggled (GObject *self, GParamSpec* pspec, gpointer user_data)
+cb_xfwm4_sync_label_link_activated (GtkLabel *label)
 {
-    /* Set the new XFWM4 theme */
-    if (gtk_switch_get_active (GTK_SWITCH (self)))
-        theme_selection_changed (GTK_TREE_SELECTION (user_data), "/Net/ThemeName");
-    else
-        xfconf_channel_set_string (xfconf_channel_get ("xfwm4"), "/general/theme", "Default");
+    gchar  *command;
+    GError *error = NULL;
+
+    command = g_find_program_in_path ("xfwm4-settings");
+    if (command != NULL && !g_spawn_command_line_async (command, &error))
+    {
+         xfce_dialog_show_error (NULL, error, _("Failed to display Xfwm4 Settings"));
+         g_error_free (error);
+    }
+
+    g_free (command);
 }
 
 static void
@@ -826,33 +832,16 @@ appearance_settings_dialog_channel_property_changed (XfconfChannel *channel,
             g_free (new_name);
         }
 
-        /* Keep gsettings in sync */
+        /* Set the preferred color scheme (needed for GTK4) */
         if (desktop_interface_gsettings != NULL)
         {
-            GSettingsSchema *schema;
-
             str = xfconf_channel_get_string (channel, property_name, NULL);
-            g_settings_set_string (desktop_interface_gsettings, "gtk-theme", str);
-
-            /* Also set the preferred color scheme (needed for GTK4) */
-            g_object_get (desktop_interface_gsettings, "settings-schema", &schema, NULL);
-            if (g_settings_schema_has_key (schema, "color-scheme"))
-              {
-                if (str != NULL)
-                {
-                    gchar *strdown = g_ascii_strdown (str, -1);
-                    if (g_strstr_len (strdown, -1, "-dark-") || g_str_has_suffix (strdown, "-dark"))
-                        g_settings_set_string (desktop_interface_gsettings, "color-scheme", "prefer-dark");
-                    else
-                        g_settings_reset (desktop_interface_gsettings, "color-scheme");
-                    g_free (strdown);
-                }
-                else
-                    g_settings_reset (desktop_interface_gsettings, "color-scheme");
-              }
+            if (str != NULL && g_str_has_suffix (str, "-dark"))
+                g_settings_set_string (desktop_interface_gsettings, "color-scheme", "prefer-dark");
+            else
+                g_settings_reset (desktop_interface_gsettings, "color-scheme");
 
             g_free (str);
-            g_settings_schema_unref (schema);
         }
     }
     else if (strcmp (property_name, "/Net/IconThemeName") == 0)
@@ -895,24 +884,6 @@ appearance_settings_dialog_channel_property_changed (XfconfChannel *channel,
                                  pd,
                                  (GDestroyNotify) preview_data_free);
         }
-
-        /* Keep gsettings in sync */
-        if (desktop_interface_gsettings != NULL)
-        {
-            str = xfconf_channel_get_string (channel, property_name, NULL);
-            g_settings_set_string (desktop_interface_gsettings, "icon-theme", str);
-            g_free (str);
-        }
-    }
-    else if (strcmp (property_name, "/Gtk/FontName") == 0)
-    {
-        /* Keep gsettings in sync */
-        if (desktop_interface_gsettings != NULL)
-        {
-            str = xfconf_channel_get_string (channel, property_name, NULL);
-            g_settings_set_string (desktop_interface_gsettings, "font-name", str);
-            g_free (str);
-        }
     }
     else if (strcmp (property_name, "/Gtk/MonospaceFontName") == 0)
     {
@@ -1276,8 +1247,10 @@ appearance_settings_dialog_configure_widgets (GtkBuilder *builder)
     if (path != NULL)
     {
         object = gtk_builder_get_object (builder, "xfwm4_sync_switch");
-        xfconf_g_property_bind (xsettings_channel, "/Xfce/SyncThemes", G_TYPE_BOOLEAN, G_OBJECT (object), "active");
-        g_signal_connect (G_OBJECT (object), "notify::active", G_CALLBACK (cb_xfwm4_sync_switch_toggled), selection);
+        xfconf_g_property_bind (xsettings_channel, "/Xfce/SyncThemes", G_TYPE_BOOLEAN, G_OBJECT (object), "state");
+
+        object = gtk_builder_get_object (builder, "xfwm4_sync_label");
+        g_signal_connect (G_OBJECT (object), "activate-link", G_CALLBACK (cb_xfwm4_sync_label_link_activated), NULL);
     }
     else
     {
@@ -1436,7 +1409,7 @@ main (gint argc, gchar **argv)
     if (G_UNLIKELY (opt_version))
     {
         g_print ("%s %s (Xfce %s)\n\n", G_LOG_DOMAIN, PACKAGE_VERSION, xfce_version_string ());
-        g_print ("%s\n", "Copyright (c) 2008-2024");
+        g_print ("%s\n", "Copyright (c) 2008-2023");
         g_print ("\t%s\n\n", _("The Xfce development team. All rights reserved."));
         g_print (_("Please report bugs to <%s>."), PACKAGE_BUGREPORT);
         g_print ("\n");
