@@ -501,8 +501,6 @@ xfce_displays_helper_get_matching_profile (XfceDisplaysHelper *helper)
         xfsettings_dbg (XFSD_DEBUG_DISPLAYS, "Found %d matching display profiles.", g_list_length (profiles));
     }
 
-    g_list_free_full (profiles, g_free);
-
     return NULL;
 }
 
@@ -529,7 +527,9 @@ xfce_displays_helper_screen_on_event (GdkXEvent *xevent,
 
     event_num = e->type - helper->event_base;
 
-    if (event_num == RRScreenChangeNotify)
+    autoconnect_mode = xfconf_channel_get_int (helper->channel, NOTIFY_PROP, 1);
+
+    if (event_num == RRScreenChangeNotify && autoconnect_mode > 0)
     {
         xfsettings_dbg (XFSD_DEBUG_DISPLAYS, "RRScreenChangeNotify event received.");
 
@@ -539,15 +539,12 @@ xfce_displays_helper_screen_on_event (GdkXEvent *xevent,
         xfsettings_dbg (XFSD_DEBUG_DISPLAYS, "Noutput: before = %d, after = %d.",
                         old_outputs->len, helper->outputs->len);
 
-        autoconnect_mode = xfconf_channel_get_int (helper->channel, NOTIFY_PROP, 1);
-
         /* Check if we have different amount of outputs and a matching profile and
            apply it if there's only one */
         if (old_outputs->len > helper->outputs->len ||
             old_outputs->len < helper->outputs->len)
         {
-            if (xfconf_channel_get_bool (helper->channel, AUTO_ENABLE_PROFILES, FALSE)
-                && autoconnect_mode > 0)
+            if (xfconf_channel_get_bool (helper->channel, AUTO_ENABLE_PROFILES, FALSE))
             {
                 gchar *matching_profile = NULL;
 
@@ -635,8 +632,8 @@ xfce_displays_helper_screen_on_event (GdkXEvent *xevent,
                             crtc->rotation = RR_Rotate_0;
 G_GNUC_BEGIN_IGNORE_DEPRECATIONS
                             if ((crtc->x > gdk_screen_width() + 1) || (crtc->y > gdk_screen_height() + 1)
-                                || autoconnect_mode == 2)
-                            {
+                                || autoconnect_mode == 2) {
+G_GNUC_END_IGNORE_DEPRECATIONS
                                 crtc->x = crtc->y = 0;
                             }
                             /* Extend to the right if configured */
@@ -645,7 +642,6 @@ G_GNUC_BEGIN_IGNORE_DEPRECATIONS
                                 crtc->x = helper->width - crtc->width;
                                 crtc->y = 0;
                             } /* else - leave values from last time we saw the monitor */
-G_GNUC_END_IGNORE_DEPRECATIONS
                             /* set width and height */
                             for (j = 0; j < helper->resources->nmode; ++j)
                             {
@@ -669,7 +665,8 @@ G_GNUC_END_IGNORE_DEPRECATIONS
 
             /* Start the minimal dialog according to the user preferences */
             if (changed && autoconnect_mode == 1)
-                xfce_spawn_command_line (NULL, "xfce4-display-settings -m", FALSE, FALSE, TRUE, NULL);
+                xfce_spawn_command_line_on_screen (NULL, "xfce4-display-settings -m", FALSE,
+                                                   FALSE, NULL);
         }
         g_ptr_array_unref (old_outputs);
     }
@@ -914,7 +911,7 @@ xfce_displays_helper_load_from_xfconf (XfceDisplaysHelper *helper,
                         helper->resources->modes[m].height);
 
             /* find the mode corresponding to the saved values */
-            if (rint (rate * 100) == rint (output_rate * 100)
+            if (rint (rate * 10) == rint (output_rate * 10)
                 && (g_strcmp0 (property, str_value) == 0))
             {
                 valid_mode = helper->resources->modes[m].id;
@@ -999,6 +996,8 @@ xfce_displays_helper_list_outputs (XfceDisplaysHelper *helper)
 
     g_assert (XFCE_IS_DISPLAYS_HELPER (helper) && helper->xdisplay && helper->resources);
 
+    outputs = g_ptr_array_new ();
+
     /* get all connected outputs */
     outputs = g_ptr_array_new_with_free_func ((GDestroyNotify) xfce_displays_helper_free_output);
     for (n = 0; n < helper->resources->noutput; ++n)
@@ -1536,7 +1535,7 @@ xfce_displays_helper_channel_apply (XfceDisplaysHelper *helper,
     /* safety check */
     if (nactive == 0)
     {
-        g_warning ("Stored Xfconf properties disable all outputs, aborting.");
+        g_critical ("Stored Xfconf properties disable all outputs, aborting.");
         goto err_cleanup;
     }
 
